from dartrs.dartrs import DartTokenizer


def test_load_tokenizer():
    tokenizer = DartTokenizer.from_pretrained("p1atdev/dart-v2-mixtral-160m-sft-2")
    assert tokenizer is not None


def test_encode_text():
    tokenizer = DartTokenizer.from_pretrained("p1atdev/dart-v2-mixtral-160m-sft-2")

    text = "1girl, cat ears"
    encoded = tokenizer.encode(text)

    assert encoded is not None
    assert len(encoded) > 0
    assert encoded == [103, 255]


def test_decode_text():
    tokenizer = DartTokenizer.from_pretrained("p1atdev/dart-v2-mixtral-160m-sft-2")

    text = "1girl, cat ears</general><|eos|>"
    encoded = tokenizer.encode(text)

    decoded = tokenizer.decode(encoded)

    assert decoded is not None
    assert decoded == "1girl, cat ears"


def test_decode_tags():
    tokenizer = DartTokenizer.from_pretrained("p1atdev/dart-v2-mixtral-160m-sft-2")

    text = "1girl, cat ears</general><|eos|>"
    encoded = tokenizer.encode(text)

    decoded = tokenizer.decode_tags(encoded)

    assert decoded is not None
    assert decoded == ["1girl", "cat ears"]


def test_decode_tags_text_with_special_tokens():
    tokenizer = DartTokenizer.from_pretrained("p1atdev/dart-v2-mixtral-160m-sft-2")

    text = "1girl, cat ears</general><|eos|>"
    encoded = tokenizer.encode(text)

    decoded = tokenizer.decode_tags(encoded, skip_special_tokens=False)

    assert decoded is not None
    assert decoded == ["1girl", "cat ears", "</general>", "<|eos|>"]


def test_tokenize_text():
    tokenizer = DartTokenizer.from_pretrained("p1atdev/dart-v2-mixtral-160m-sft-2")

    text = "1girl, cat ears, hogeeee"
    tokens = tokenizer.tokenize(text)

    assert tokens is not None
    assert tokens == ["1girl", "cat ears", "<|unk|>"]



from typing import Literal, Generator

from . import dartrs


LengthTag = (
    Literal["very_short"]
    | Literal["short"]
    | Literal["medium"]
    | Literal["long"]
    | Literal["very_long"]
)

AspectRatioTag = (
    Literal["ultra_wide"]
    | Literal["wide"]
    | Literal["square"]
    | Literal["tall"]
    | Literal["ultra_tall"]
)

RatingTag = (
    Literal["sfw"]
    | Literal["general"]
    | Literal["sensitive"]
    | Literal["nsfw"]
    | Literal["questionable"]
    | Literal["explicit"]
)

IdentityTag = Literal["none"] | Literal["lax"] | Literal["strict"]


def compose_prompt(
    prompt: str = "",
    copyright: str = "",
    character: str = "",
    rating: RatingTag = "general",
    aspect_ratio: AspectRatioTag = "tall",
    length: LengthTag = "medium",
    identity: IdentityTag = "none",
    do_completion: bool = True,
):
    return dartrs.compose_prompt_v2(
        copyright=copyright,
        character=character,
        rating=dartrs.RatingTag(f"<|rating:{rating}|>"),
        aspect_ratio=dartrs.AspectRatioTag(f"<|aspect_ratio:{aspect_ratio}|>"),
        length=dartrs.LengthTag(f"<|length:{length}|>"),
        identity_level=dartrs.IdentityTag(f"<|identity:{identity}|>"),
        prompt=prompt,
        do_completion=do_completion,
    )


class V2Model:
    model: dartrs.DartV2Mistral | dartrs.DartV2Mixtral

    def __init__(
        self,
        model: dartrs.DartV2Mistral | dartrs.DartV2Mixtral,
    ) -> None:
        self.model = model

    def generate(self, config: dartrs.GenerationConfig) -> str:
        """Generates tags."""
        return self.model.generate(config)

    def _get_next_token(
        self, config: dartrs.GenerationConfig, cache: dartrs.GenerationCache
    ) -> tuple[int, dartrs.GenerationCache]:
        """Generates the next token."""
        return self.model.get_next_token(config, cache)

    def generate_stream(self, config: dartrs.GenerationConfig) -> Generator[
        str,  # tag
        None,
        str,  # final decoded text
    ]:
        """Generates tags and returns the final decoded text."""

        tokens = config.tokenizer().encode(config.prompt())
        cache = dartrs.GenerationCache(tokens)

        for _ in range(0, config.max_new_tokens()):
            token, cache = self._get_next_token(config, cache)
            tag = config.tokenizer().decode([token], skip_special_tokens=True)
            yield tag

            if cache.finished():
                break

        self.model._clear_kv_cache()  # clear kv cache

        decoded = config.tokenizer().decode(
            cache.output_tokens(), skip_special_tokens=True
        )
        return decoded


class MixtralModel(V2Model):
    @classmethod
    def from_pretrained(
        cls,
        hub_name: str,
        revision: str | None = None,
        dtype: dartrs.DartDType = dartrs.DartDType.FP32,
        device: dartrs.DartDevice = dartrs.DartDevice.Cpu(),
        auth_token: str | None = None,
    ) -> V2Model:
        return cls(dartrs.DartV2Mixtral(hub_name, revision, dtype, device, auth_token))


class MistralModel(V2Model):
    @classmethod
    def from_pretrained(
        cls,
        hub_name: str,
        revision: str | None = None,
        dtype: dartrs.DartDType = dartrs.DartDType.FP32,
        device: dartrs.DartDevice = dartrs.DartDevice.Cpu(),
        auth_token: str | None = None,
    ) -> V2Model:
        return cls(dartrs.DartV2Mistral(hub_name, revision, dtype, device, auth_token))
